<?php
// $Id: mode.admin.inc,v 1.1.2.1 2009/02/27 20:29:25 karthik Exp $

/**
 * Menu callback: Mode module settings form.
 */
function mode_list() {
  $header = array(t('Name'), t('Status'), array('data' => t('Operations'), 'colspan' => 3));
  $rows = array();
  $modes = mode_modes_load();

  foreach ($modes as $mode) {
    $rows[] = array(
      check_plain($mode->name),
      $mode->active ? t('Enabled') : t('Disabled'),
      l(t('Edit'), 'admin/settings/mode/edit/'. $mode->mid),
      l(t('Delete'), 'admin/settings/mode/delete/'. $mode->mid),
      $mode->active ? l(t('Reapply'), 'admin/settings/mode/switch/'. $mode->mid) : l(t('Apply'), 'admin/settings/mode/switch/'. $mode->mid)
    );
  }

  return theme_table($header, $rows);
}

/**
 * Menu callback: Mode settings form.
 */
function mode_settings_form($form_state) {
  $mode_config = _mode_variable_get();

  $form['mode_config'] = array('#tree' => TRUE);
  $form['mode_config']['banner'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display banner message'),
    '#description' => t('If enabled, the banner message associated with a mode will be displayed on all pages as a status message.'),
    '#default_value' => $mode_config['banner']
  );
  $form['mode_config']['permissions_page'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disable permissions page'),
    '#description' => t('If enabled, the permissions page will be disabled along with a message stating that the mode settings page should be used instead.'),
    '#default_value' => $mode_config['permissions_page']
  );

  return system_settings_form($form);
}

/**
 * Menu callback: Mode add/edit form.
 */
function mode_edit_form($form_state, $mode_id = NULL) {
  $mode = $mode_id ? mode_modes_load($mode_id) : NULL;

  require_once(drupal_get_path('module', 'user') .'/user.admin.inc');
  $form = user_admin_perm(NULL);
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Give this mode a unique name for identification purposes.'),
    '#required' => TRUE,
    '#weight' => 1
  );
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#description' => t('Status message to display on all pages when this mode is enabled. HTML can be used. A toggle switch is also available on the settings page to control display.'),
    '#rows' => 2,
    '#weight' => 2
  );
  // Due to the way this form is rendered in theme_user_admin_perm, we shove the
  // name field to the bottom, but above the submit button.
  $form['submit']['#weight'] = 3;

  if ($mode) {
    drupal_set_title(t('Mode edit: %name', array('%name' => $mode->name)));

    $form['name']['#default_value'] = $mode->name;
    $form['message']['#default_value'] = $mode->message;
    foreach ($form['checkboxes'] as $rid => $options) {
      $defaults = isset($mode->data[$rid]) ? explode(', ', $mode->data[$rid]) : array();
      $form['checkboxes'][$rid]['#default_value'] = $defaults;
    }
    $form['mid'] = array('#type' => 'value', '#value' => $mode->mid);

    $form['submit']['#value'] = t('Update mode');
  }
  else {
    $form['submit']['#value'] = t('Add mode');
  }

  return $form;
}

/**
 * Process mode edit form submissions.
 */
function mode_edit_form_submit($form, &$form_state) {
  $form_values = $form_state['values'];
  $permissions = array();

  $result = db_query('SELECT * FROM {role}');
  while ($role = db_fetch_object($result)) {
    if (isset($form_values[$role->rid])) {
      $form_values[$role->rid] = array_filter($form_values[$role->rid]);
      if (count($form_values[$role->rid])) {
        $permissions[$role->rid] = implode(', ', array_keys($form_values[$role->rid]));
      }
    }
  }

  $mode = array('name' => trim($form_values['name']), 'data' => serialize($permissions), 'message' => trim($form_values['message']));
  $update = array();
  if (isset($form_values['mid'])) {
    $mode['mid'] = $form_values['mid'];
    $update[] = 'mid';
  }
  $status = mode_mode_add($mode, $update);

  $form_state['redirect'] = 'admin/settings/mode';
}

/**
 * Delete a mode.
 */
function mode_delete_form($form_state, $mode_id = NULL) {
  $mode = $mode_id ? mode_modes_load($mode_id) : NULL;
  $form['mode'] = array('#type' => 'value', '#value' => $mode);
  return confirm_form($form,
    t('Are you sure you want to delete the mode %name?', array('%name' => $mode->name)),
    'admin/settings/mode',
    t('This will delete the mode permanently. This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Process mode delete form submissions.
 */
function mode_delete_form_submit($form, &$form_state) {
  mode_mode_delete($form_state['values']['mode']);
  $form_state['redirect'] = 'admin/settings/mode';
}

/**
 * Switch to a different mode.
 *
 * @param Integer $mode
 *   The mode ID of the mode to the switched to.
 * @param Boolean $notify
 *   Displays status messages to the user only if set to TRUE. 
 */
function mode_modes_switch($mode, $notify = TRUE) {
  $mode = mode_modes_load($mode);
  if (isset($mode->mid)) {
    db_query('DELETE FROM {permission}');
    foreach ($mode->data as $rid => $permissions) {
      $data = array('rid' => $rid, 'perm' => $permissions);
      $status = drupal_write_record('permission', $data);
    }
    // Only going to bother with the status of the last record.
    if ($status == SAVED_NEW) {
      // Set all modes to disabled.
      db_query('UPDATE {mode} SET active = 0');
      db_query('UPDATE {mode} SET active = 1 WHERE mid = %d', $mode->mid);
      $message = 'Mode %name enabled.';
      $vars = array('%name' => $mode->name);
      watchdog('user', $message, $vars);
      if ($notify) {
        drupal_set_message(t($message, $vars));
      }
    }

    cache_clear_all();
    drupal_goto('admin/settings/mode');
  }
}
